#!perl

# CVS: $Id: ljpms.pl,v 2.5 2005/01/03 12:19:08 sasha Exp $
# Author: Alexander Nikolaev <sasha_nikolaev@yahoo.com>
# Descr: change security level of all posts exported in XML by ljsm.pl
# 		import previously exported journal; optionally change journal name
# 
# cyrillic encodings language pack for XML::Parser:
# http://uucode.com/xml/perl/enc.zip
#
# des-itemid: Itemid URL appears in. For non-clustered users, this is just
# the itemid. For clustered users, this is the display itemid,
# which is the jitemid*256+anum from the [dbtable[log2]] table.
#
# I assume that jitemid = floor(des-itemid/256) 
# 
use Digest::MD5 qw(md5_hex);
use Getopt::Std;
use XML::Simple;
use XMLRPC::Lite
	on_fault	=> sub {
		my ($soap, $res) = @_;
		if (ref $res) {
			warn "\n--- LIVEJOURNAL FAULT ---\n", 
				$res->faultcode, ": ",
				$res->faultstring, "\n";
		} else {
			warn "\n--- HTTP ERROR ---\n", 
			$soap->transport->status, "\n";
		}
		return new SOAP::SOM;
	}
;

# explicitly include some modules for perl2exe
use SOAP::Transport::HTTP;

#use Data::Dumper;
#use diagnostics;
use strict;

use constant URL		=> 'http://www.livejournal.com/interface/xmlrpc';
use constant LOCALDIR	=> '';
use constant CLIENT		=> 'Perl-ljpms/1.2; sasha_nikolaev@yahoo.com';
use constant MAXTRY		=> 5;
use constant VERSION	=> '$Revision: 2.5 $';

if ($] >= 5.008) {
	require Encode;
	import Encode qw(encode_utf8);
}


# -----------------------------------------------------------

my ($user, $suser, $password, $mode, %modes, %opts, @files, $file, $xi, $entry);
my ($security, $allowmask, $res, @server, @data, $soap_message, $soap_method);
my ($year, $month, $day, $hour, $min);

%modes = (
		'private'	=> 'private',
		'friends'	=> 'usemask',
		'public'	=> 'public',
		'restore'	=> 1,
		'delete'	=> 'private'
		);

getopts('p:s:', \%opts);

# set proxy URL for LWP requests
if ($opts{'p'}) {
	$opts{'p'} = 'http://'.$opts{'p'} unless ($opts{'p'} =~ m{^http://});
	@server = (URL, 'proxy' => ['http' => $opts{'p'}]);
} else {
	@server = (URL);
}

exit(usage()) unless @ARGV == 2;
($user, $password) = split(':', $ARGV[0], 2);
$suser = (defined($opts{'s'}) && length($opts{'s'}))? $opts{'s'} : $user;

exit(usage('nopassword')) unless $password;
$mode = lc($ARGV[1]);
exit(usage('invalidmode')) unless ($modes{$mode} || $mode =~ /^\d+$/);
exit (usage('noexport')) unless -d LOCALDIR . "$suser/export";

if ($suser ne $user) { # post to someone else's journal
	exit usage('invalidmode') if ($mode eq 'delete');
	exit usage('invalidmode') if ($mode =~ /^\d+$/);
	print "posting ${suser}'s backdated entries to ${user}'s journal\n";
	$soap_method = 'postevent';
} else {
	$soap_method = 'editevent';
}

$password = md5_hex($password);

if ($mode eq 'delete') {
	warn <<EOM;
--------------------  WARNING!!! --------------------------
      The journal of $user will be erased now.
      You will NOT be able to restore it. Never.
Press Ctrl-C in 5 seconds if you don't want this to happen
-----------------------------------------------------------
EOM
	sleep 5;
}

# get list of xml files to process
opendir(UD, LOCALDIR . $suser . '/export') or 
	die "#Error opening " . LOCALDIR . "$suser/export directory for reading: $!\n";
@files = sort grep(/^\d{4}_\d{2}\.xml$/, readdir(UD));
closedir UD;
die "empty $suser/export directory, nothing to do.\n" unless @files;
print "processing ", scalar @files, " file";
print ((@files > 1)? "s\n" : "\n");


# login to server
# 
print "Logging in to server as $user... ";
$res = XMLRPC::Lite
	->proxy(@server)
	->call('LJ.XMLRPC.login', {
		'username'		=> $user,
		'hpassword'		=> $password,
		'clientversion'	=> CLIENT })
	->result();

# $responce is XML::RPC::struct
if ($res) {
	print " done.\n";
	print "Hello, " . $res->{'fullname'} . "\n";
	print $res->{'message'} if defined $res->{'message'};
	print "\n";
	
} else {
	exit 1;
}


if ($mode =~ /^\d+$/) {
	$security = 'usemask';
	$allowmask = $mode;
} else {
	$security = $modes{$mode};
	$allowmask = ($security eq 'usemask')? 1 : 0;
}	

FILES:
foreach $file (@files) {
	$xi = XMLin(LOCALDIR . "$suser/export/$file");
	@data = ();
	if (! defined $xi->{'entry'}) {
		@data = ();
	} elsif (ref $xi->{'entry'} eq 'HASH') {
		@data = ($xi->{'entry'});
	} elsif (ref $xi->{'entry'} eq 'ARRAY') {
		@data = @{$xi->{'entry'}};
	}
	
	foreach $entry (@data) {

		my $subject = '';
		if ($mode eq 'delete') {
			$subject = '';
		} else {
			$subject = my_encode($entry->{'subject'});
		}
		
		
		($year, $month, $day, $hour, $min) = split(/\D+/, my_encode($entry->{'eventtime'}));
		print $entry->{'itemid'} . " (" . substr($subject, 0, 52) . "...) ->   1 ";
		
		#set required access mode
		if ($mode eq 'restore') {
			$security = $entry->{'security'};
			$allowmask = $entry->{'allowmask'};
		}
		
		$soap_message =
			{
				'username'		=> $user,
				'hpassword'		=> $password,
				'ver'			=> 1,
				'lineendings'	=> "\n",
				
				'event'		=> ($mode eq 'delete')? '' : my_encode($entry->{'event'}),
				'subject'	=> $subject,
				'security'	=> $security,
				'allowmask'	=> $allowmask,
				
				'year'		=> $year,
				'mon'		=> $month,
				'day'		=> $day,
				'hour'		=> $hour,
				'min'		=> $min
			};

		$soap_message->{'itemid'} = ($entry->{'itemid'} >> 8)
			if ($soap_method eq 'editevent');

		$soap_message->{'props'} = {
			# not exported by export.bml:
			# -----------------------------
			# opt_preformated
			# opt_nocomments
			# opt_noemail
			# opt_screening
			# picture_keyword
			#
			'current_music'	=> my_encode($entry->{'current_music'}),
			'current_mood'	=> my_encode($entry->{'current_mood'}),
			'opt_backdated'	=> 1
		} if ($soap_method eq 'postevent');

		TRY:
		for (1 .. MAXTRY) {
			print " $_ " if ($_ > 1);
			$res = XMLRPC::Lite
				->proxy(@server)
				->call('LJ.XMLRPC.' . $soap_method, $soap_message)
				->result();
				
			if ($res) {
				print " ok\n";
				last TRY;
				
			} elsif ($_ == MAXTRY) {
				print " FAILED!\n";
				last FILES;
			}
			sleep 2;
		}
	}
}


sub my_encode {
	my ($str) = @_;
	return '' if (ref $str);
	return  ($] >= 5.008)? encode_utf8($str) : $str;
}

sub usage {
	my ($error) = @_;
	$user = 'USER' unless defined $user;
	my %errmsg = (
		'nopassword' 	=> 'use username:password as a first argument',
		'invalidmode'	=> 'mode is one of: private,friends,public,restore,delete,<usemask>',
		'noexport'		=> LOCALDIR . "$suser/export directory should contain xml files exported by ljsm\n#ERROR: (use ljsm -X -u $suser\:password)"
	);
	if ($error) {
		warn "#ERROR: $errmsg{$error}\n";
		
	} else {
		warn "ljpms - utility for batch modification of LiveJournal posts security level\n";
		warn "usage: $0 [-p proxy] [-s source_user] username:password mode\n";
		warn "-s source_user: post source_user's backdated entries to username's journal\n";
		warn "mode: private|friends|public|restore|delete|<usemask>\n";
		warn "<usemask> is a friendgroup identifyer\n\n";
		warn VERSION . "\n";
	}
	1;
}
